package pinwindows

import (
	rl "github.com/gen2brain/raylib-go/raylib"
)

type managewindowfunc func(*RayGuiWindowManager, *RayGuiWindow)

type RayGuiWindowManager struct {
	Winlist          []*RayGuiWindow
	Movingwindow     *RayGuiWindow // the currently moving (resize, drag, etc...)
	Movingwindowmode int
}

func NewRayGuiWindowManager() RayGuiWindowManager {
	var windowlist []*RayGuiWindow
	return RayGuiWindowManager{Winlist: windowlist, Movingwindow: nil}
}

func (winman *RayGuiWindowManager) ManageWindowlist() {
	var windowtomovetofront *RayGuiWindow = nil
	for i := 0; i < len(winman.Winlist); i++ {
		if winman.Winlist[i].isLeftClicked() {
			windowtomovetofront = winman.Winlist[i]
		}
	}
	if windowtomovetofront != nil {
		windowtomovetofront.ManageFunc = MoveToTopOfWindowlist
	}
	for i := 0; i < len(winman.Winlist); i++ {
		if winman.Winlist[i].ManageFunc != nil {
			winman.Winlist[i].ManageFunc(winman, winman.Winlist[i])
		}
	}
}

func (winman *RayGuiWindowManager) ProcessUserWindowMovement() {
	currentmovingwindow := winman.Movingwindow

	for i := 0; i < len(winman.Winlist); i++ {
		if winman.Winlist[i].DidStartMoveDrag() && currentmovingwindow == nil {
			winman.Movingwindow = winman.Winlist[i]
			winman.Movingwindowmode = WINDOW_MODE_DRAG
		}
		if winman.Winlist[i].DidStartResizeDrag() && currentmovingwindow == nil {
			winman.Movingwindow = winman.Winlist[i]
			winman.Movingwindowmode = WINDOW_MODE_RESZIE
		}
		if rl.IsMouseButtonReleased(rl.MouseButtonLeft) || rl.IsMouseButtonReleased(rl.MouseButtonRight) {
			winman.Movingwindow = nil
		}

	}
	if winman.Movingwindow != nil {
		if winman.Movingwindowmode == WINDOW_MODE_DRAG {
			winman.Movingwindow.Pos.X += (rl.GetMouseDelta().X)
			winman.Movingwindow.Pos.Y += (rl.GetMouseDelta().Y)
		}
		if winman.Movingwindowmode == WINDOW_MODE_RESZIE {
			winman.Movingwindow.Resize(int(rl.GetMouseDelta().X), int(rl.GetMouseDelta().Y))
		}
	}
}

func (winman RayGuiWindowManager) DrawInternalWindowList() {
	for i := 0; i < len(winman.Winlist); i++ {
		winman.Winlist[i].DrawInternals()
	}
}

func (winman RayGuiWindowManager) DrawWindowlist() {
	for i := 0; i < len(winman.Winlist); i++ {
		winman.Winlist[i].Draw()
	}
}

func (winman *RayGuiWindowManager) WinlistUnlistWindow(windowtoremove *RayGuiWindow) {
	indextoremove := -1
	for i := 0; i < len(winman.Winlist); i++ {
		if winman.Winlist[i] == windowtoremove {
			indextoremove = i
			break
		}
	}
	if indextoremove == -1 {
		return
	}
	copy(winman.Winlist[indextoremove:], winman.Winlist[indextoremove+1:])
	winman.Winlist[len(winman.Winlist)-1] = nil
	winman.Winlist = winman.Winlist[:len(winman.Winlist)-1]
	return
}

func CloseWindow(winman *RayGuiWindowManager, windowtoremove *RayGuiWindow) {
	winman.WinlistUnlistWindow(windowtoremove)
	rl.UnloadRenderTexture(*windowtoremove.Texture)
	if winman.Movingwindow == windowtoremove {
		winman.Movingwindow = nil
	}
	return
}

func (winman *RayGuiWindowManager) AddWindow(windowtoadd *RayGuiWindow) {
	if len(winman.Winlist) > 0 {
		winman.Winlist[len(winman.Winlist)-1].AcceptingInput = false
	}
	winman.Winlist = append(winman.Winlist, windowtoadd)
	winman.Winlist[len(winman.Winlist)-1].AcceptingInput = true
}

func MoveToTopOfWindowlist(winman *RayGuiWindowManager, windowtomove *RayGuiWindow) {
	winman.Winlist[len(winman.Winlist)-1].AcceptingInput = false
	winman.WinlistUnlistWindow(windowtomove)
	winman.AddWindow(windowtomove)
	windowtomove.ManageFunc = nil
}
